@@ -23,8 +23,8 @@
 
 #include "gkr-pkcs11-auth.h"
 
+#include "egg/egg-cleanup.h"
 #include "egg/egg-secure-memory.h"
-#include "common/gkr-cleanup.h"
 
 #include "keyrings/gkr-keyring-login.h"
 
@@ -126,12 +126,16 @@ prepare_specific_secondary (CK_OBJECT_CLASS klass, const gchar *label)
 {
 	switch (klass) {
 	case CKO_PRIVATE_KEY:
+		/* TRANSLATORS: The private key is locked */
 		return g_strdup_printf (_("An application wants access to the private key '%s', but it is locked"), label);
 	case CKO_CERTIFICATE:
+		/* TRANSLATORS: The certificate is locked */
 		return g_strdup_printf (_("An application wants access to the certificate '%s', but it is locked"), label);
 	case CKO_PUBLIC_KEY:
+		/* TRANSLATORS: The public key is locked */
 		return g_strdup_printf (_("An application wants access to the public key '%s', but it is locked"), label);
 	default:
+		/* TRANSLATORS: The object '%s' is locked */
 		return g_strdup_printf (_("An application wants access to '%s', but it is locked"), label);
 	}
 }
@@ -173,6 +177,13 @@ gkr_pkcs11_auth_login_specific_prepare (CK_SESSION_HANDLE handle, GkrPkcs11AuthO
 	g_hash_table_replace (slot->session_to_specific, ulong_alloc (handle), object);
 }
 
+static void
+convert_upper_case (gchar *str)
+{
+	for (; *str; ++str)
+		*str = g_ascii_toupper (*str);
+}
+
 gboolean 
 gkr_pkcs11_auth_login_specific_prompt (CK_SESSION_HANDLE handle, CK_SESSION_INFO *info,
                                        CK_UTF8CHAR_PTR *pin, CK_ULONG *pin_len)
@@ -212,7 +223,7 @@ gkr_pkcs11_auth_login_specific_prompt (CK_SESSION_HANDLE handle, CK_SESSION_INFO
 	}
 	
 	/* See if we can just use the login keyring password for this */
-	if (object->unique && object->token && gkr_keyring_login_is_usable ()) {
+	if (object->unique && object->token) {
 		password = gkr_keyring_login_lookup_secret (GNOME_KEYRING_ITEM_ENCRYPTION_KEY_PASSWORD,
 		                                            "unique", object->unique, NULL);
 		if (password != NULL) { 
@@ -220,6 +231,21 @@ gkr_pkcs11_auth_login_specific_prompt (CK_SESSION_HANDLE handle, CK_SESSION_INFO
 			return TRUE;
 		}
 	}
+	
+	/* COMPAT: Check old method of storing secrets for objects in login keyring */
+	if (object->digest) {
+		convert_upper_case (object->digest);
+		password = gkr_keyring_login_lookup_secret (GNOME_KEYRING_ITEM_PK_STORAGE,
+		                                            "object-digest", object->digest, NULL);
+		if (password != NULL) {
+			if (object->unique)
+				gkr_keyring_login_attach_secret (GNOME_KEYRING_ITEM_ENCRYPTION_KEY_PASSWORD, 
+				                                 object->label, password, 
+		                                                 "unique", object->unique, NULL);
+			password_to_pin (password, pin, pin_len);
+			return TRUE;
+		}
+	}
 
 	/* Build up the prompt */
 	flags = GKR_ASK_REQUEST_PASSWORD | GKR_ASK_REQUEST_OK_DENY_BUTTONS;
@@ -367,10 +393,10 @@ gkr_pkcs11_auth_login_user_prompt (CK_SESSION_HANDLE handle, CK_TOKEN_INFO *info
 	ask = gkr_ask_request_new (_("Unlock certificate/key storage"), 
 	                           _("Enter password to unlock the certificate/key storage"), flags); 
 	
+	/* TRANSLATORS: The storage is locked, and needs unlocking before the application can use it. */
 	secondary = g_strdup_printf (_("An application wants access to the certificate/key storage '%s', but it is locked"), label);
 	gkr_ask_request_set_secondary (ask, secondary);
 	g_free (secondary);
-	g_free (label);
 	
 	if (gkr_keyring_login_is_usable ())
 		gkr_ask_request_set_check_option (ask, _("Automatically unlock secure storage when I log in."));
